#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Use function when converting into ONNX
# 
# Once a a scikit-learn model is converting into ONNX, there is no easy way to retrieve the original scikit-learn model. The following notebook explores an alternative way to convert a model into ONNX by using functions. In this new method, every piece of a pipeline becomes a function.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# In[2]:


get_ipython().run_line_magic('matplotlib', 'inline')


# In[3]:


get_ipython().run_line_magic('load_ext', 'mlprodict')


# ## A pipeline

# In[4]:


from sklearn.pipeline import Pipeline
from sklearn.datasets import load_iris
from sklearn.preprocessing import StandardScaler
from sklearn.linear_model import LogisticRegression
from sklearn import set_config
set_config(display="diagram")

data = load_iris()
X, y = data.data, data.target
steps = [
    ("preprocessing", StandardScaler()),
    ("classifier", LogisticRegression(penalty='l1', solver="liblinear"))]
pipe = Pipeline(steps)
pipe.fit(X, y)


# ## Its conversion into ONNX

# ### Without functions

# In[5]:


from mlprodict.plotting.text_plot import onnx_simple_text_plot
from mlprodict.onnx_conv import to_onnx

onx = to_onnx(pipe, X, options={'zipmap': False})
print(onnx_simple_text_plot(onx))


# In[6]:


get_ipython().run_line_magic('onnxview', 'onx')


# ### With functions

# In[7]:


onxf = to_onnx(pipe, X, as_function=True, options={'zipmap': False})
print(onnx_simple_text_plot(onxf))


# In[8]:


get_ipython().run_line_magic('onnxview', 'onxf')


# Based on that, it should be possible to rebuild the original scikit-learn pipeline. Hyperparameters are stored in the attribute  `doc_string`.

# ## A more complex one

# In[9]:


from sklearn.compose import ColumnTransformer
from sklearn.preprocessing import MinMaxScaler

data = load_iris()
X, y = data.data, data.target
steps = [
    ("preprocessing", ColumnTransformer([
        ('A', StandardScaler(), [0, 1]),
        ('B', MinMaxScaler(), [2, 3])])),
    ("classifier", LogisticRegression(penalty='l1', solver="liblinear"))]
pipe = Pipeline(steps)
pipe.fit(X, y)


# In[10]:


onxf = to_onnx(pipe, X, as_function=True, options={'zipmap': False})
print(onnx_simple_text_plot(onxf))


# In[11]:


get_ipython().run_line_magic('onnxview', 'onxf')


# In[12]: